@@ -1,4 +1,5 @@
-#!/usr/bin/python -u
+#!/usr/bin/env python3
+import setup_test
 import libxml2
 
 # Memory debug specific
@@ -29,4 +30,3 @@ if libxml2.debugMemory(1) == 0:
     print("OK")
 else:
     print("Memory leak %d bytes" % (libxml2.debugMemory(1)))
-    libxml2.dumpMemory()
