@@ -1,5 +1,6 @@
-#!/usr/bin/python -u
+#!/usr/bin/env python3
 #
+import setup_test
 import libxml2
 
 expect=' xmlns:a="urn:whatevar"'
@@ -11,6 +12,7 @@ d = libxml2.parseDoc("<a:a xmlns:a='urn:whatevar'/>")
 res=""
 for n in d.xpathEval("//namespace::*"):
     res = res + n.serialize()
+del n
 d.freeDoc()
 
 if res != expect:
@@ -18,7 +20,6 @@ if res != expect:
     print(res)
 del res
 del d
-del n
 # Memory debug specific
 libxml2.cleanupParser()
 
@@ -26,4 +27,3 @@ if libxml2.debugMemory(1) == 0:
     print("OK")
 else:
     print("Memory leak %d bytes" % (libxml2.debugMemory(1)))
-    libxml2.dumpMemory()
